/* 
 * Copyright (C) 2005-2005 Virus27, Virus27@free.fr
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "stdinc.h"
#include "DCPlusPlus.h"

#include "RawManager.h"

#include "ResourceManager.h"
#include "version.h"
#include "SimpleXML.h"
#include "ClientManager.h"

void RawManager::saveActionRaws() {
	try {
		SimpleXML xml;
		xml.addTag("ActionRaws");
		xml.addChildAttrib("Version", string(VERSIONSTRING));
		xml.stepIn();

		for(Action::Iter l = action.begin(); l != action.end(); ++l) {
			xml.addTag("Action");
			xml.addChildAttrib("ID", Util::toString(l->second->getActionId()));
			xml.addChildAttrib("Name", l->second->getName());
			xml.addChildAttrib("Active", Util::toString(l->second->getActive()));
			xml.stepIn();
			for(Action::RawIter j = l->second->raw.begin(); j != l->second->raw.end(); ++j) {
				xml.addTag("Raw");
				xml.addChildAttrib("ID", Util::toString(j->getRawId()));
				xml.addChildAttrib("Name", j->getName());
				xml.addChildAttrib("Raw", j->getRaw());
				xml.addChildAttrib("Time", Util::toString(j->getTime()));
				xml.addChildAttrib("Active", Util::toString(j->getActive()));
			}
			xml.stepOut();
		}
		xml.stepOut();

		string fname = Util::getConfigPath() + "Raws.xml";

		File f(fname + ".tmp", File::WRITE, File::CREATE | File::TRUNCATE);
		f.write(SimpleXML::utf8Header);
		f.write(xml.toXML());
		f.close();
		File::deleteFile(fname);
		File::renameFile(fname + ".tmp", fname);

	} catch(const Exception& e) {
		dcdebug("RawManager::saveActionRaws: %s\n", e.getError().c_str());
	}
}

void RawManager::loadActionRaws(SimpleXML* aXml) {
	aXml->resetCurrentChild();

	while(aXml->findChild("Action")) {
		int actionId(aXml->getIntChildAttrib("ID"));
		string name(aXml->getChildAttrib("Name"));
		bool active(getVersion().empty() ? aXml->getBoolChildAttrib("Actif") : aXml->getBoolChildAttrib("Active"));
		addAction(actionId, name, active);
		aXml->stepIn();
		while(aXml->findChild("Raw")) {
			int rawId(aXml->getIntChildAttrib("ID"));
			string nameRaw(aXml->getChildAttrib("Name"));
			string raw(aXml->getChildAttrib("Raw"));
			string time(aXml->getChildAttrib("Time"));
			bool activeRaw(getVersion().empty() ? aXml->getBoolChildAttrib("Actif") : aXml->getBoolChildAttrib("Active"));
			addRaw(actionId, rawId, nameRaw, raw, Util::toInt(time), activeRaw);
		}
		aXml->stepOut();
	}
}

void RawManager::loadActionRaws() {
	// Try to import raws from other clients
	if(!Util::fileExists(Util::getConfigPath() + "Raws.xml")) {
		importRaws();
		return;
	}

	try {
		SimpleXML xml;
		xml.fromXML(File(Util::getConfigPath() + "Raws.xml", File::READ, File::OPEN).read());
		
		if(xml.findChild("ActionRaws")) {
			setVersion(xml.getChildAttrib("Version"));
			xml.stepIn();
			loadActionRaws(&xml);
			xml.stepOut();
		}
	} catch(const Exception& e) {
		dcdebug("RawManager::loadActionRaws: %s\n", e.getError().c_str());
	}
}

int RawManager::addAction(int actionId, const string& name, bool active) throw(Exception) {
	if(name.empty())
		throw Exception(STRING(NO_NAME_SPECIFIED));

	Lock l(act);

	for(Action::Iter i = action.begin(); i != action.end(); ++i) {
		if(i->second->getName() == name)
			throw Exception(STRING(ACTION_EXISTS));
	}

	while(actionId == 0) {
		actionId = Util::rand(1, 2147483647);
		for(Action::Iter i = action.begin(); i != action.end(); ++i) {
			if(i->second->getActionId() == actionId) {
				actionId = 0;
			}
		}
	}
	int id = lastAction++;

	action.insert(make_pair(id, new Action(actionId, name, active)));

	return id;
}

void RawManager::renameAction(const string& oName, const string& nName) throw(Exception) {
	if(oName.empty() || nName.empty())
		throw Exception(STRING(NO_NAME_SPECIFIED));

	Lock l(act);

	for(Action::Iter i = action.begin(); i != action.end(); ++i) {
		if(i->second->getName() == nName)
			throw Exception(STRING(ACTION_EXISTS));
	}
	for(Action::Iter i = action.begin(); i != action.end(); ++i) {
		if(i->second->getName() == oName)
			i->second->setName(nName);
	}
}

void RawManager::setActiveAction(int id, bool active) {
	Lock l(act);
	Action::Iter i;
	if((i = action.find(id)) != action.end()) {
		i->second->setActive(active);
	}
}

bool RawManager::getActiveActionId(int actionId)  {
	Lock l(act);
	for(Action::Iter i = action.begin(); i != action.end(); ++i) {
		if(i->second->getActionId() == actionId) {
			return i->second->getActive();
		}
	}
	return false;
}

void RawManager::removeAction(int id) {
	Lock l(act);
	Action::List::iterator i;
	if((i = action.find(id)) != action.end()) {
		delete i->second;
		action.erase(i);
	}
}

int RawManager::getValidAction(int actionId) {
	Lock l(act);
	for(Action::Iter i = action.begin(); i != action.end(); ++i) {
		if(i->second->getActionId() == actionId)
			return i->second->getActionId();
	}
	return 0;
}

int RawManager::getActionId(int id) {
	Lock l(act);
	Action::Iter i;
	if((i = action.find(id)) != action.end()) {
		return i->second->getActionId();
	}
	return 0;
}

tstring RawManager::getNameActionId(int actionId) {
	Lock l(act);
	for(Action::Iter i = action.begin(); i != action.end(); ++i) {
		if(i->second->getActionId() == actionId)
			return Text::toT(i->second->getName());
	}
	return TSTRING(NO_ACTION);
}

Action::RawList RawManager::getRawList(int id) {
	Lock l(act);

	Action::Iter i;
	Action::RawList lst;
	if((i = action.find(id)) != action.end()) {
		lst = i->second->raw;
	}
	return lst;
}

Action::RawList RawManager::getRawListActionId(int actionId) {
	Lock l(act);
	
	Action::RawList lst;
	for(Action::Iter i = action.begin(); i != action.end(); ++i) {
		if(i->second->getActionId() == actionId)
			lst = i->second->raw;
	}
	return lst;
}

void RawManager::addRaw(int actionId, int rawId, const string& name, const string& raw, int time, bool active) {
	if(name.empty())
		return;

	Lock l(act);

	for(Action::Iter i = action.begin(); i != action.end(); ++i) {
		if(i->second->getActionId() == actionId) {

			for(Action::RawIter j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
				if(j->getName() == name)
					return;
			}

			while(rawId == 0) {
				rawId = Util::rand(1, 2147483647);
				for(Action::RawIter j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
					while(j->getRawId() == rawId) {
						rawId = 0;
					}
				}
			}

			i->second->raw.push_back(
				Action::Raw(
				i->second->lastRaw++,
				rawId,
				name,
				raw,
				time,
				active
				)
			);
			return;
		}
	}
}

Action::Raw RawManager::addRaw(int id, const string& name, const string& raw, int time) throw(Exception) {
	if(name.empty())
		throw Exception(STRING(NO_NAME_SPECIFIED));

	Lock l(act);

	Action::Iter i;
	if((i = action.find(id)) != action.end()) {
		for(Action::RawIter j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
			if(j->getName() == name)
				throw Exception(STRING(RAW_EXISTS));
		}

		int rawId = 0;
		while(rawId == 0) {
			rawId = Util::rand(1, 2147483647);
			for(Action::RawIter j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
				while(j->getRawId() == rawId) {
					rawId = 0;
				}
			}
		}

		i->second->raw.push_back(
			Action::Raw(
			i->second->lastRaw++,
			rawId,
			name,
			raw,
			time,
			true
			)
		);
		return i->second->raw.back();
	}
	return Action::Raw(0, 0, Util::emptyString, Util::emptyString, 0, false);
}

void RawManager::changeRaw(int id, const string& oName, const string& nName, const string& raw, int time) throw(Exception) {
	if(oName.empty() || nName.empty())
		throw Exception(STRING(NO_NAME_SPECIFIED));

	Lock l(act);

	Action::Iter i;
	if((i = action.find(id)) != action.end()) {
		if(oName != nName) {
			for(Action::RawIter j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
				if(j->getName() == nName)
					throw Exception(STRING(RAW_EXISTS));
			}
		}

		for(Action::RawList::iterator j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
			if(j->getName() == oName) {
				j->setName(nName);
				j->setRaw(raw);
				j->setTime(time);
				break;
			}
		}
	}
}

void RawManager::getRawItem(int id, int idRaw, Action::Raw& raw, bool favHub/* = false*/) {
	Lock l(act);
	Action::Iter i;
	if((i = action.find(id)) != action.end()) {
		for(Action::RawIter j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
			if (favHub) {
				if(j->getRawId() == idRaw) {
					raw = *j;
					break;
				}
			} else {
				if(j->getId() == idRaw) {
					raw = *j;
					break;
				}
			}
		}
	}
}

void RawManager::setActiveRaw(int id, int idRaw, bool active) {
	Lock l(act);
	Action::Iter i;
	if((i = action.find(id)) != action.end()) {
		for(Action::RawList::iterator j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
			if(j->getId() == idRaw) {
				j->setActive(active);
				break;
			}
		}
	}
}

void RawManager::removeRaw(int id, int idRaw) {
	Lock l(act);
	Action::Iter i;
	if((i = action.find(id)) != action.end()) {
		for(Action::RawList::iterator j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
			if(j->getId() == idRaw) {
				i->second->raw.erase(j);
				break;
			}
		}
	}
}

void RawManager::importRaws() {
	try {
		vector<int> actionList;
		SimpleXML xml;

		xml.fromXML(File(Util::getConfigPath() + "DCPlusPlus.xml", File::READ, File::OPEN).read());
		xml.resetCurrentChild();
		xml.stepIn();
		if(xml.findChild(("Settings"))) {
			xml.stepIn();
			if(xml.findChild("RawOneText")) { actionList.push_back(addAction(0, (xml.getChildData().empty() ? "Raw 1" : xml.getChildData()), true)); }
			xml.resetCurrentChild();
			if(xml.findChild("RawTwoText")) { actionList.push_back(addAction(0, (xml.getChildData().empty() ? "Raw 2" : xml.getChildData()), true)); }
			xml.resetCurrentChild();
			if(xml.findChild("RawThreeText")) { actionList.push_back(addAction(0, (xml.getChildData().empty() ? "Raw 3" : xml.getChildData()), true)); }
			xml.resetCurrentChild();
			if(xml.findChild("RawFourText")) { actionList.push_back(addAction(0, (xml.getChildData().empty() ? "Raw 4" : xml.getChildData()), true)); }
			xml.resetCurrentChild();
			if(xml.findChild("RawFiveText")) { actionList.push_back(addAction(0, (xml.getChildData().empty() ? "Raw 5" : xml.getChildData()), true)); }
			xml.resetCurrentChild();
		}
		xml.resetCurrentChild();
		xml.stepOut();

		xml.fromXML(File(Util::getConfigPath() + "Favorites.xml", File::READ, File::OPEN).read());
		xml.resetCurrentChild();
		xml.stepIn();
		if(xml.findChild("Hubs")) {
			xml.stepIn();
			while(xml.findChild("Hub")) {
				string name, r1, r2, r3, r4, r5;

				name = xml.getChildAttrib("Name");
				r1 = xml.getChildAttrib("RawOne");
				r2 = xml.getChildAttrib("RawTwo");
				r3 = xml.getChildAttrib("RawThree");
				r4 = xml.getChildAttrib("RawFour");
				r5 = xml.getChildAttrib("RawFive");

				if(!name.empty()) {
					if(!r1.empty()) addRaw(getActionId(actionList.at(0)), 0, name, r1, 0, true);
					if(!r2.empty()) addRaw(getActionId(actionList.at(1)), 0, name, r2, 0, true);
					if(!r3.empty()) addRaw(getActionId(actionList.at(2)), 0, name, r3, 0, true);
					if(!r4.empty()) addRaw(getActionId(actionList.at(3)), 0, name, r4, 0, true);
					if(!r5.empty()) addRaw(getActionId(actionList.at(4)), 0, name, r5, 0, true);
				}
			}
			xml.resetCurrentChild();
			xml.stepOut();
		}
	} catch(...) {
		// ...
	}
}

bool RawManager::moveRaw(int id, int idRaw, int pos) {
	dcassert(pos == -1 || pos == 1);
	Lock l(act);
	Action::Iter i;
	if((i = action.find(id)) != action.end()) {
		for(Action::RawList::iterator j = i->second->raw.begin(); j != i->second->raw.end(); ++j) {
			if(j->getId() == idRaw) {
				swap(*j, *(j + pos));
				return true;
			}
		}
	}
	return false;
}

void RawManager::on(TimerManagerListener::Second, uint64_t aTick) throw() {
	Lock l(act);
	for(IterRaw i = raw.begin(); i != raw.end(); ++i) {
		if(aTick >= i->first) {
			ClientManager::getInstance()->sendRawCommand(i->second.user, i->second.raw);
			raw.erase(i);
		}
	}
}